var Istat = function()
{
	var istat = {};
	var debug = 0;
	
	var REPORT_TYPE_STATISTICS = 'statistics';
	var REPORT_TYPE_STATISTICS_CSV = 'statistics_csv';
	var incaFilesAreValid = false;
	var incaFileStructuresAreValid = false;
	var jsonIncaData = {};

	istat.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', istat.validateStep1);
		Events.addEventHandler('incafile', 'change', istat.incaFileOnChange);
		Buttons.addClickHandler('gostatistics', istat.calcStatistics);
		Buttons.addClickHandler('simplecheck', istat.simpleCheckOfIncaFile);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('downloadreportfile', istat.downloadReportFile);
		Buttons.addClickHandler('downloadstatcsvfile', istat.downloadStatCsvFile);

/*
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', istat.submit);
*/

		// Initialize INCA export date to current date
		var frm = document.forms['reggie'];
		frm.startDate.value = '20100101';
		frm.endDate.value = Dates.formatDate(new Date(), 'yyyyMMdd');
		
		Doc.show('step-1');
		Doc.show('gostatistics');
		Doc.addClass('gostatistics', 'disabled');
		Doc.show('simplecheck');
		Doc.addClass('simplecheck', 'disabled');
		// If report file exists, show button to download it
		istat.checkForReportFile(REPORT_TYPE_STATISTICS);
		// If CSV file exists, show button to download it
		istat.checkForReportFile(REPORT_TYPE_STATISTICS_CSV);
	}
	
	istat.validateStep1 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('incafile');
		if (!valid) event.preventDefault();
	}

	istat.incaFileOnChange = function()
	{
		var frm = document.forms['reggie'];
		var files = frm.incafile.files;

		if (files != null)
		{
			Doc.removeClass('gostatistics', 'disabled');
			Doc.removeClass('simplecheck', 'disabled');
		}
		else
		{
			Doc.addClass('gostatistics', 'disabled');
			Doc.addClass('simplecheck', 'disabled');
		}
	}

	istat.incaFileLoaded = function(event)
	{
		Doc.removeClass('gostatistics', 'disabled');
		Doc.removeClass('simplecheck', 'disabled');
	}

	istat.simpleCheckOfIncaFile = function()
	{
		var frm = document.forms['reggie'];
		var statDateVar = frm.statisticsDateVar.value;
		var startDate = frm.startDate.value;
		var endDate = frm.endDate.value;
		var cancerType = frm.cancerType.value;

		Doc.addClass('gostatistics', 'disabled');
		Doc.addClass('simplecheck', 'disabled');

		var formData = new FormData();
		var files = frm.incafile.files;
		for (var i = 0; i < files.length; i++)
		{
			var file = files[i];
			formData.append('incafiles[]', file, file.name);
		}

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=IncaStatistics';
		url += '&statdatevar='+encodeURIComponent(statDateVar);
		url += '&startdate='+encodeURIComponent(startDate);
		url += '&enddate='+encodeURIComponent(endDate);
		url += '&cancertype='+encodeURIComponent(cancerType);
		url += '&simplecheckonly=true';

		// POST
		Wizard.showLoadingAnimation('Checking INCA data...', 'inca-stat-progress');
		Wizard.asyncJsonRequest(url, istat.initializeStep2, 'POST', formData);
	}

	istat.calcStatistics = function()
	{
		var frm = document.forms['reggie'];
		var statDateVar = frm.statisticsDateVar.value;
		var startDate = frm.startDate.value;
		var endDate = frm.endDate.value;
		var cancerType = frm.cancerType.value;

		Doc.addClass('gostatistics', 'disabled');
		Doc.addClass('simplecheck', 'disabled');
		Doc.addClass('downloadreportfile', 'disabled');
		Doc.addClass('downloadstatcsvfile', 'disabled');

		var formData = new FormData();
		var files = frm.incafile.files;
		for (var i = 0; i < files.length; i++)
		{
			var file = files[i];
			formData.append('incafiles[]', file, file.name);
		}

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=IncaStatistics';
		url += '&statdatevar='+encodeURIComponent(statDateVar);
		url += '&startdate='+encodeURIComponent(startDate);
		url += '&enddate='+encodeURIComponent(endDate);
		url += '&cancertype='+encodeURIComponent(cancerType);
		url += '&calcstatistics=true';

		// POST
		Wizard.showLoadingAnimation('Checking INCA data...', 'inca-stat-progress');
		Wizard.asyncJsonRequest(url, istat.initializeStep2, 'POST', formData);
	}

	istat.initializeStep2 = function(response)
	{
		var frm = document.forms['reggie'];
		Wizard.setCurrentStep(2);

		// Get INCA file properties from JSON data
		
		// INCA file properties - basic check
		var fileProp = response.incaFileProperties;
		var incaStat = response.incaStatistics;
		var simpleCheckOnly = response.simpleCheckOnly;

		// Print name of file
		var filelisttext = '';
		var tmpFilename = fileProp.filename;
		filelisttext += '<b>&nbsp;File:&nbsp;' + tmpFilename + '</b><br>';
		Doc.element('incafilenames').innerHTML = filelisttext;

		// Print date variable to use for statistics and time period
		var statDateVar = fileProp.statDateVar;
		var starDateVarDescr = 'Diagnosis date';
		if (statDateVar == 'operation')
		{
			starDateVarDescr = 'Operation date';
		}
		var statdatedescrtext = '<b>&nbsp;Statistics date variable:&nbsp;' + starDateVarDescr + '</b><br>';
		Doc.element('statDateDescr').innerHTML = statdatedescrtext;

		var statDateClmHeader = fileProp.statDateClmHeader;
		var statdateclmtext = '<b>&nbsp;Statistics date column header:&nbsp;' + statDateClmHeader + '</b><br>';
		Doc.element('statDateClmHeader').innerHTML = statdateclmtext;

		// Print cancer type filter
		var cancerType = fileProp.cancerType;
		var cancerTypeDescr = 'All';
		if (cancerType == 'invasive')
		{
			cancerTypeDescr = 'Invasive';
		}
		else if (cancerType == 'insitu')
		{
			cancerTypeDescr = 'In situ';
		}
		var cancertypedescrtext = '<b>&nbsp;Cancer type:&nbsp;' + cancerTypeDescr + '</b><br>';
		Doc.element('cancerTypeSelected').innerHTML = cancertypedescrtext;

		//
		// INCA file properties - Basic check
		//
		var basicchecktabletext = '<table class="incafileproptable">';

		// Table header
		basicchecktabletext += istat.createTableHeader();

		// # Header columns
		var numHeaderColumnsOk = istat.fetchTableRowStatus(fileProp, 'numHeaderColumns', 'lt', 3);
		basicchecktabletext += istat.createTableRow(fileProp, '# Header columns', 'numHeaderColumns', numHeaderColumnsOk, 'error', '&#8805; 3 allowed');

		// # Lines of data
		var numLinesOk = istat.fetchTableRowStatus(fileProp, 'numLines', 'lt', 1);
		basicchecktabletext += istat.createTableRow(fileProp, '# Lines of data', 'numLines', numLinesOk, 'error', '');

		// # Lines with internal line feeds
		var numLinesWithLineFeedsOk = istat.fetchTableRowStatus(fileProp, 'numLinesWithLineFeeds', 'gt', 0);
		basicchecktabletext += istat.createTableRow(fileProp, '# Lines with internal line feeds', 'numLinesWithLineFeeds', numLinesWithLineFeedsOk, 'warning', '');

		// # Lines with too many columns
		var numLinesWithTooManyColumnsOk = istat.fetchTableRowStatus(fileProp, 'numLinesWithTooManyColumns', 'gt', 0);
		basicchecktabletext += istat.createTableRow(fileProp, '# Lines with too many columns', 'numLinesWithTooManyColumns', numLinesWithTooManyColumnsOk, 'error', '0 allowed');

		// # Lines with too few columns
		var numLinesWithTooFewColumnsOk = istat.fetchTableRowStatus(fileProp, 'numLinesWithTooFewColumns', 'gt', 0);
		basicchecktabletext += istat.createTableRow(fileProp, '# Lines with too few columns', 'numLinesWithTooFewColumns', numLinesWithTooFewColumnsOk, 'error', '0 allowed');

		//
		basicchecktabletext += '</table>';
		Doc.element('incaFilePropBasicCheckTable').innerHTML = basicchecktabletext;

		//
		// INCA file properties - Internal data check
		//
		var internalchecktabletext = '<table class="incafileproptable">';

		// Table header
		internalchecktabletext += istat.createTableHeader();

		// # Duplicate header columns
		var numHeaderDuplicatesOk = istat.fetchTableRowStatus(fileProp, 'numHeaderDuplicates', 'gt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Duplicate header columns', 'numHeaderDuplicates', numHeaderDuplicatesOk, 'error', '0 allowed');

		// Statistics date column index
		var statDateClmIndexOk = istat.fetchTableRowStatus(fileProp, 'statDateClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Statistics date column index', 'statDateClmIndex', statDateClmIndexOk, 'error', '&#8805; 0 allowed');

		// Cancer type column index
		var cancerTypeClmIndexOk = istat.fetchTableRowStatus(fileProp, 'cancerTypeClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Cancer type column index', 'cancerTypeClmIndex', cancerTypeClmIndexOk, 'error', '&#8805; 0 allowed');

		// Temporary patient ID column index
		var tempPatIdClmIndexOk = istat.fetchTableRowStatus(fileProp, 'tempPatIdClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Temporary patient ID column index', 'tempPatIdClmIndex', tempPatIdClmIndexOk, 'error', '&#8805; 0 allowed');

		// Personal number column index
		var personalNoClmIndexOk = istat.fetchTableRowStatus(fileProp, 'personalNoClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Personal number column index', 'personalNoClmIndex', personalNoClmIndexOk, 'error', '&#8805; 0 allowed');

		// Laterality description column index
		var lateralityDescriptionClmIndexOk = istat.fetchTableRowStatus(fileProp, 'lateralityDescriptionClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Laterality description column index', 'lateralityDescriptionClmIndex', lateralityDescriptionClmIndexOk, 'error', '&#8805; 0 allowed');

		// ER status column index
		var erStatusClmIndexOk = istat.fetchTableRowStatus(fileProp, 'erStatusClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'ER status column index', 'erStatusClmIndex', erStatusClmIndexOk, 'error', '&#8805; 0 allowed');

		// PgR column index
		var pgrClmIndexOk = istat.fetchTableRowStatus(fileProp, 'pgrClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'PgR column index', 'pgrClmIndex', pgrClmIndexOk, 'error', '&#8805; 0 allowed');

		// HER2 status column index
		var her2StatusClmIndexOk = istat.fetchTableRowStatus(fileProp, 'her2StatusClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'HER2 status column index', 'her2StatusClmIndex', her2StatusClmIndexOk, 'error', '&#8805; 0 allowed');

		// Age (years) column index
		var ageYearsClmIndexOk = istat.fetchTableRowStatus(fileProp, 'ageYearsClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Age (years) column index', 'ageYearsClmIndex', ageYearsClmIndexOk, 'error', '&#8805; 0 allowed');

		// NHG column index
		var nhgClmIndexOk = istat.fetchTableRowStatus(fileProp, 'nhgClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'NHG column index', 'nhgClmIndex', nhgClmIndexOk, 'error', '&#8805; 0 allowed');

		// Tumor size (mm) column index
		var tumorSizeMmInSituClmIndexOk = istat.fetchTableRowStatus(fileProp, 'tumorSizeMmInSituClmIndex', 'lt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, 'Tumor size (mm) column index', 'tumorSizeMmInSituClmIndex', tumorSizeMmInSituClmIndexOk, 'error', '&#8805; 0 allowed');

/*
		// # Unknown headers
		var numUnknownHeadersOk = istat.fetchTableRowStatus(fileProp, 'numUnknownHeaders', 'gt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Unknown headers', 'numUnknownHeaders', numUnknownHeadersOk, 'warning', 'Columns skipped');
*/

		// # Data lines passing time period filter
		var numTimePeriodOkLinesOk = istat.fetchTableRowStatus(fileProp, 'numTimePeriodOkLines', 'eq', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Data lines passing time period filter', 'numTimePeriodOkLines', numTimePeriodOkLinesOk, 'error', 'Required for statistics');

		// # Data lines passing cancer type filter
		var numCancerTypeOkLinesOk = istat.fetchTableRowStatus(fileProp, 'numCancerTypeOkLines', 'eq', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Data lines passing cancer type filter', 'numCancerTypeOkLines', numCancerTypeOkLinesOk, 'error', 'Required for statistics');

		// # Data lines with personal no.
		var numPersonalNoLinesOk = istat.fetchTableRowStatus(fileProp, 'numPersonalNoLines', 'eq', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Data lines with personal no.', 'numPersonalNoLines', numPersonalNoLinesOk, 'error', 'Required for statistics');

		// # Personal no. with more than 2 lines
		var numPersonalNoWithoutLateralityOk = iport.fetchTableRowStatus(fileProp, 'numPersonalNoWithoutLaterality', 'gt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Personal no. without laterality', 'numPersonalNoWithoutLaterality', numPersonalNoWithoutLateralityOk, 'warning', 'Counted as accrued without specimen');

		// # Personal no.s with many identical lateralities
		var numPersonalNoWithManyIdenticalLateralityLinesOk = istat.fetchTableRowStatus(fileProp, 'numPersonalNoWithManyIdenticalLateralityLines', 'gt', 0);
		internalchecktabletext += istat.createTableRow(fileProp, '# Personal no.s with many identical lateralities', 'numPersonalNoWithManyIdenticalLateralityLines', numPersonalNoWithManyIdenticalLateralityLinesOk, 'warning', 'Counted as accrued without specimen');

		//
		internalchecktabletext += '</table>';
		Doc.element('incaFilePropInternalDataCheckTable').innerHTML = internalchecktabletext;

		if (numHeaderColumnsOk &&
			numLinesOk &&
			numLinesWithTooManyColumnsOk &&
			numLinesWithTooFewColumnsOk &&
			numHeaderDuplicatesOk &&
			statDateClmIndexOk &&
			cancerTypeClmIndexOk &&
			tempPatIdClmIndexOk &&
			personalNoClmIndexOk &&
			lateralityDescriptionClmIndexOk &&
			numPersonalNoLinesOk)
		{
			incaFileStructuresAreValid = true;
		}

		//
		// INCA file properties - Database consistency check
		//
		if (!simpleCheckOnly)
		{
			var consistencychecktabletext = '<table class="incafileproptable">';

			// Table header
			consistencychecktabletext += istat.createTableHeader();

			// # Data lines with personal no. not in database
			var numDataLinesWithoutDatabaseReferenceOk = istat.fetchTableRowStatus(fileProp, 'numDataLinesWithoutDatabaseReference', 'gt', 0);
			consistencychecktabletext += istat.createTableRow(fileProp, '# Data lines with personal no. not in database', 'numDataLinesWithoutDatabaseReference', numDataLinesWithoutDatabaseReferenceOk, 'warning', 'Counted as accrued without specimen');

			// # Patient lateralities without database reference
			var numPatientLateralitiesWithoutDatabaseReferenceOk = istat.fetchTableRowStatus(fileProp, 'numPatientLateralitiesWithoutDatabaseReference', 'gt', 0);
			consistencychecktabletext += istat.createTableRow(fileProp, '# Patient lateralities without database reference', 'numPatientLateralitiesWithoutDatabaseReference', numPatientLateralitiesWithoutDatabaseReferenceOk, 'warning', 'Counted as accrued without specimen');

			// # Data lines with bad values
			var numBadValueLinesOk = istat.fetchTableRowStatus(fileProp, 'numBadValueLines', 'gt', 0);
			consistencychecktabletext += istat.createTableRow(fileProp, '# Data lines with bad values', 'numBadValueLines', numBadValueLinesOk, 'warning', 'Added to N/A fraction for column with bad value');

			//
			consistencychecktabletext += '</table>';
			Doc.element('incaFilePropDatabaseConsistencyCheckTable').innerHTML = consistencychecktabletext;
		}
		else
		{
			var consistencychecktabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaFilePropDatabaseConsistencyCheckTable').innerHTML = consistencychecktabletext;
		}

		//
		// INCA statistics - Total
		//
		if (!simpleCheckOnly)
		{
			var jsonStatTotal = incaStat.jsonStatTotal;
			var accruedWithSpecPct = jsonStatTotal.accruedWithSpecPct;
			var accruedNoSpecPct = jsonStatTotal.accruedNoSpecPct;
			var nonAccruedPct = jsonStatTotal.nonAccruedPct;

			var totalstattabletext = '<table class="incafileproptable">';

			// Table header
			totalstattabletext += istat.createTotalStatTableHeader();

			// % Accrued and tumor specimen
			//totalstattabletext += istat.createStatTableRow('Accrued and tumor specimen', jsonStatTotal, 'accruedWithSpecPct', null, null, '');
			totalstattabletext += istat.createStatTableRow('Accrued and tumor specimen', jsonStatTotal, 'accruedWithSpecPct', 'accruedWithSpecNum', null, null, '');

			// % Accrued but no tumor specimen
			//totalstattabletext += istat.createStatTableRow('Accrued but no tumor specimen', jsonStatTotal, 'accruedNoSpecPct', null, null, '');
			totalstattabletext += istat.createStatTableRow('Accrued but no tumor specimen', jsonStatTotal, 'accruedNoSpecPct', 'accruedNoSpecNum', null, null, '');

			// % Not accrued
			//totalstattabletext += istat.createStatTableRow('Not accrued', jsonStatTotal, 'nonAccruedPct', null, null, '');
			totalstattabletext += istat.createStatTableRow('Not accrued', jsonStatTotal, 'nonAccruedPct', 'nonAccruedNum', null, null, '');

			//
			totalstattabletext += '</table>';
			var numTotal = jsonStatTotal.numTotal;
			var totalstattabletitle = '<b>&nbsp;INCA statistics - Total (n = ' + numTotal + ' eliglible diagnoses)</b>';
			Doc.element('incaStatisticsTotalStatTableTitle').innerHTML = totalstattabletitle;
			Doc.element('incaStatisticsTotalStatTable').innerHTML = totalstattabletext;
		}
		else
		{
			var totalstattabletitle = '<b>&nbsp;INCA statistics - Total</b>';
			var totalstattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsTotalStatTableTitle').innerHTML = totalstattabletitle;
			Doc.element('incaStatisticsTotalStatTable').innerHTML = totalstattabletext;
		}

		//
		// INCA statistics - ER Status
		//
		if (!simpleCheckOnly)
		{
			var erstattabletext = istat.createPosNegStatusStatTableText(incaStat.jsonErStat);
			Doc.element('incaStatisticsErStatTable').innerHTML = erstattabletext;
		}
		else
		{
			var erstattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsErStatTable').innerHTML = erstattabletext;
		}

		//
		// INCA statistics - PgR Status
		//
		if (!simpleCheckOnly)
		{
			var pgrstattabletext = istat.createPosNegStatusStatTableText(incaStat.jsonPgrStat);
			Doc.element('incaStatisticsPgrStatTable').innerHTML = pgrstattabletext;
		}
		else
		{
			var pgrstattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsPgrStatTable').innerHTML = pgrstattabletext;
		}

		//
		// INCA statistics - HER2 Status
		//
		if (!simpleCheckOnly)
		{
			var her2stattabletext = istat.createPosNegStatusStatTableText(incaStat.jsonHer2Stat);
			Doc.element('incaStatisticsHer2StatTable').innerHTML = her2stattabletext;
		}
		else
		{
			var her2stattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsHer2StatTable').innerHTML = her2stattabletext;
		}

		//
		// INCA statistics - Age (years)
		//
		if (!simpleCheckOnly)
		{
			var agestattabletext = istat.createGroupStatTableText(incaStat.jsonAgeStat);
			Doc.element('incaStatisticsAgeStatTable').innerHTML = agestattabletext;
		}
		else
		{
			var agestattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsAgeStatTable').innerHTML = agestattabletext;
		}

		//
		// INCA statistics - NHG
		//
		if (!simpleCheckOnly)
		{
			var nhgstattabletext = istat.createGradeStatTableText(incaStat.jsonNhgStat);
			Doc.element('incaStatisticsNhgStatTable').innerHTML = nhgstattabletext;
		}
		else
		{
			var nhgstattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsNhgStatTable').innerHTML = nhgstattabletext;
		}

		//
		// INCA statistics - Tumor Size (mm)
		//
		if (!simpleCheckOnly)
		{
			var tumorsizestattabletext = istat.createGroupStatTableText(incaStat.jsonTumorSizeMmInSituStat);
			Doc.element('incaStatisticsTumorSizeStatTable').innerHTML = tumorsizestattabletext;
		}
		else
		{
			var tumorsizestattabletext = '&nbsp;(Not included in simple file check)';
			Doc.element('incaStatisticsTumorSizeStatTable').innerHTML = tumorsizestattabletext;
		}

		Doc.show('step-2');

		// Show summary line
		var text = "";
/*
		if (!simpleCheckOnly)
		{
			var numDataLinesForImport = fileProp.numDataLinesForImport;
			numPersonalNoLines = fileProp.numPersonalNoLines;
			text += "&nbsp;# Data lines for import = " + numDataLinesForImport + " of " + numPersonalNoLines + "<br><br>";
		}
*/
		if (!incaFileStructuresAreValid)
		{
			text += "<img src=\"../images/error.png\" style=\"padding-left: 1em;\" />&nbsp;Errors were found in INCA file[s]. Fix these and try again.";
		}
		Doc.element('filepropertymessage').innerHTML = text;

		if (incaFileStructuresAreValid)
		{
			Wizard.showFinalMessage(response.messages);
		}
		
		if (response.notFound && response.notFound.length)
		{
			Doc.element('not-found').innerHTML = response.notFound.join('<br>');
			Doc.show('not-found');
		}

		istat.selectOptionsDisabled('statisticsDateVar', true);
		istat.selectOptionsDisabled('cancerType', true);
		Doc.addClass('startDate', 'disabled');
		Doc.addClass('startDateCalendar', 'disabled');
		Doc.addClass('endDate', 'disabled');
		Doc.addClass('endDateCalendar', 'disabled');
		Doc.addClass('incafile', 'disabled');

		if (simpleCheckOnly)
		{
			Doc.show('gostatistics');
			Doc.removeClass('gostatistics', 'disabled');
		}
		else
		{
			Doc.show('gorestart');
		}
		Doc.hide('simplecheck');
		Doc.show('downloadreportfile');
		Doc.removeClass('downloadreportfile', 'disabled');
		if (simpleCheckOnly)
		{
			// Statistics CSV file is not updated for simple check
			Doc.hide('downloadstatcsvfile');
			Doc.addClass('downloadstatcsvfile', 'disabled');
		}
		else
		{
			Doc.show('downloadstatcsvfile');
			Doc.removeClass('downloadstatcsvfile', 'disabled');
		}
		if (!incaFileStructuresAreValid)
		{
			Doc.show('gorestart');
		}
	}

	istat.selectOptionsDisabled = function(id, value)
	{
		var op = document.getElementById(id).getElementsByTagName('option');
		for (var i = 0; i < op.length; i++)
		{
			op[i].disabled = value;
		}
	}

	istat.createTableHeader = function()
	{
		// Table header
		var text = '';
		text += '<tr>';
		text += '<th class="propertycolumn">Property</th>';
		text += '<th class="valuecolumn">Value</th>';
		text += '<th class="statuscolumn">Status</th>';
		text += '<th>Comment</th>';
		text += '</tr>';
		return text;
	}

	istat.fetchTableRowStatus = function(tmpFileProp, clmKey, failOperator, failLimit)
	{
		var valuesOk = true;
		var value = tmpFileProp[clmKey];
		if (failOperator == 'eq')
		{
			if (value == failLimit)
			{
				valuesOk = false;
			}
		}
		else if (failOperator == 'lt')
		{
			if (value < failLimit)
			{
				valuesOk = false;
			}
		}
		else if (failOperator == 'gt')
		{
			if (value > failLimit)
			{
				valuesOk = false;
			}
		}
		return valuesOk;
	}

	istat.createTableRow = function(tmpFileProp, title, clmKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var value = tmpFileProp[clmKey];
		text += '<td class="valuecolumn">' + value + '</td>';
/*
		var statusTag = istat.fetchValidityIcon(status, alertType);
		text += '<td class="statuscolumn">' + statusTag + '</td>';
*/
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.createTableRow2 = function(tmpFileProp, title, clmKey, failOperator, failLimit, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var valuesOk = true;
		var value = tmpFileProp[clmKey];
		text += '<td class="valuecolumn">' + value + '</td>';
		if (failOperator == 'eq')
		{
			if (value == failLimit)
			{
				valuesOk = false;
			}
		}
		else if (failOperator == 'lt')
		{
			if (value < failLimit)
			{
				valuesOk = false;
			}
		}
		else if (failOperator == 'gt')
		{
			if (value > failLimit)
			{
				valuesOk = false;
			}
		}
		var statusTag = istat.fetchValidityIcon(valuesOk, alertType);
		text += '<td class="statuscolumn">' + statusTag + '</td>';
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.fetchValidityIcon = function(isValid, notificationType)
	{
		var tag = "";
		if (isValid)
		{
			tag = "<img src=\"../images/ok.png\" style=\"padding-left: 1em;\" />";			
		}
		else
		{
			tag = "<img src=\"../images/" + notificationType + ".png\" style=\"padding-left: 1em;\" />";
		}
		return tag;
	}

	istat.addValidityIcon = function(id, isValid, notificationType)
	{
		if (isValid)
		{
			Doc.element(id).innerHTML = "<img src=\"../images/ok.png\" style=\"padding-left: 1em;\" />";			
		}
		else
		{
			Doc.element(id).innerHTML = "<img src=\"../images/" + notificationType + ".png\" style=\"padding-left: 1em;\" />";			
		}
	}

	istat.createTotalStatTableHeader = function()
	{
		// Table header
		var text = '';
		text += '<tr>';
		text += '<th class="propertycolumn">Fraction desc.</th>';
		text += '<th class="valuecolumn">% (number)</th>';
		text += '<th>Comment</th>';
		text += '</tr>';
		return text;
	}

	istat.createStatTableRow = function(title, jsonStat, jsonKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var rawValue = jsonStat[jsonKey];
		var value = parseFloat(rawValue).toFixed(2);
		text += '<td class="valuecolumn">' + value + '</td>';
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.createStatTableRow = function(title, jsonStat, jsonPctKey, jsonNumKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var numValue = jsonStat[jsonNumKey];
		var rawValue = jsonStat[jsonPctKey];
		var pctValue = parseFloat(rawValue).toFixed(2);
		text += '<td class="valuecolumn">' + pctValue + '% (' + numValue + ')' + '</td>';
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.createPosNegStatusStatTableText = function(jsonStat)
	{
		var jsonAll = jsonStat.all;
		var jsonAccrued = jsonStat.accrued;
		var jsonAccruedWithSpec = jsonStat.accruedWithSpec;

		var stattabletext = '<table class="incafileproptable">';

		var numAll = jsonAll.num;
		var numAccrued = jsonAccrued.num;
		var numAccruedWithSpec = jsonAccruedWithSpec.num;

		// Table header
		stattabletext += istat.createVarStatTableHeader(numAll, numAccrued, numAccruedWithSpec);

		// % Positive
		//stattabletext += istat.createVarStatTableRow('Positive', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'posPct', null, null, '');
		//stattabletext += istat.createVarStatTableRow('Positive', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'posPct', 'posNum', null, null, '');
		stattabletext += istat.createVarStatTableRow3('Positive', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'posExclPct', 'posPct', 'posNum', null, null, '');

		// % Negative
		//stattabletext += istat.createVarStatTableRow('Negative', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'negPct', null, null, '');
		//stattabletext += istat.createVarStatTableRow('Negative', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'negPct', 'negNum', null, null, '');
		stattabletext += istat.createVarStatTableRow3('Negative', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'negExclPct', 'negPct', 'negNum', null, null, '');

		// % N/A
		//stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', null, null, '');
		stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', 'naNum', null, null, '');

		//
		stattabletext += '</table>';
			
		return stattabletext;
	}

	istat.createGroupStatTableText = function(jsonStat)
	{
		var middleGroupMin = jsonStat.middleGroupMin;
		var middleGroupMax = jsonStat.middleGroupMax;
		
		var jsonAll = jsonStat.all;
		var jsonAccrued = jsonStat.accrued;
		var jsonAccruedWithSpec = jsonStat.accruedWithSpec;

		var stattabletext = '<table class="incafileproptable">';

		var numAll = jsonAll.num;
		var numAccrued = jsonAccrued.num;
		var numAccruedWithSpec = jsonAccruedWithSpec.num;

		// Table header
		stattabletext += istat.createVarStatTableHeader(numAll, numAccrued, numAccruedWithSpec);

		// % Group 3
		var group3title = '> ' + middleGroupMax;
		//stattabletext += istat.createVarStatTableRow(group3title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group3Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow(group3title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group3Pct', 'group3Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3(group3title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group3ExclPct', 'group3Pct', 'group3Num', null, null, '');

		// % Group 2
		var group2title = '' + middleGroupMin + '-' + middleGroupMax;
		//stattabletext += istat.createVarStatTableRow(group2title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group2Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow(group2title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group2Pct', 'group2Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3(group2title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group2ExclPct', 'group2Pct', 'group2Num', null, null, '');

		// % Group 1
		var group1title = '< ' + middleGroupMin;
		//stattabletext += istat.createVarStatTableRow(group1title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group1Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow(group1title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group1Pct', 'group1Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3(group1title, jsonAll, jsonAccrued, jsonAccruedWithSpec, 'group1ExclPct', 'group1Pct', 'group1Num', null, null, '');

		// % N/A
		//stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', null, null, '');
		stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', 'naNum', null, null, '');

		//
		stattabletext += '</table>';
			
		return stattabletext;
	}

	istat.createGradeStatTableText = function(jsonStat)
	{
		var jsonAll = jsonStat.all;
		var jsonAccrued = jsonStat.accrued;
		var jsonAccruedWithSpec = jsonStat.accruedWithSpec;

		var stattabletext = '<table class="incafileproptable">';

		var numAll = jsonAll.num;
		var numAccrued = jsonAccrued.num;
		var numAccruedWithSpec = jsonAccruedWithSpec.num;

		// Table header
		stattabletext += istat.createVarStatTableHeader(numAll, numAccrued, numAccruedWithSpec);

		// % Grade 3
		//stattabletext += istat.createVarStatTableRow('Grade 3', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade3Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow('Grade 3', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade3Pct', 'grade3Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3('Grade 3', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade3ExclPct', 'grade3Pct', 'grade3Num', null, null, '');

		// % Grade 2
		//stattabletext += istat.createVarStatTableRow('Grade 2', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade2Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow('Grade 2', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade2Pct', 'grade2Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3('Grade 2', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade2ExclPct', 'grade2Pct', 'grade2Num', null, null, '');

		// % Grade 1
		//stattabletext += istat.createVarStatTableRow('Grade 1', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade1Pct', null, null, '');
		//stattabletext += istat.createVarStatTableRow('Grade 1', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade1Pct', 'grade1Num', null, null, '');
		stattabletext += istat.createVarStatTableRow3('Grade 1', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'grade1ExclPct', 'grade1Pct', 'grade1Num', null, null, '');

		// % N/A
		//stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', null, null, '');
		stattabletext += istat.createVarStatTableRow('N/A', jsonAll, jsonAccrued, jsonAccruedWithSpec, 'naPct', 'naNum', null, null, '');

		//
		stattabletext += '</table>';
			
		return stattabletext;
	}

	istat.createVarStatTableHeader = function(numAll, numAccrued, numAccruedWithSpecimen)
	{
		// Table header
		var text = '';
		text += '<tr>';
		text += '<th class="propertycolumn">Fraction desc.</th>';
		text += '<th class="valuecolumn">All Diagnoses<br>(n='+numAll+')</th>';
		text += '<th class="valuecolumn">Accrued<br>(n='+numAccrued+')</th>';
		text += '<th class="valuecolumn">w/ Specimen<br>(n='+numAccruedWithSpecimen+')</th>';
		text += '<th>Comment</th>';
		text += '</tr>';
		return text;
	}

	istat.createVarStatTableRow = function(title, jsonAll, jsonAccrued, jsonAccruedWithSpec, jsonKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var allRawValue = jsonAll[jsonKey];
		var accruedRawValue = jsonAccrued[jsonKey];
		var accruedWithSpecRawValue = jsonAccruedWithSpec[jsonKey];
		var allValue = parseFloat(allRawValue).toFixed(2);
		var accruedValue = parseFloat(accruedRawValue).toFixed(2);
		var accruedWithSpecValue = parseFloat(accruedWithSpecRawValue).toFixed(2);
		text += '<td class="valuecolumn">' + allValue + '</td>';
		text += '<td class="valuecolumn">' + accruedValue + '</td>';
		text += '<td class="valuecolumn">' + accruedWithSpecValue + '</td>';
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.createVarStatTableRow = function(title, jsonAll, jsonAccrued, jsonAccruedWithSpec, jsonPctKey, jsonNumKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var allNumValue = jsonAll[jsonNumKey];
		var accruedNumValue = jsonAccrued[jsonNumKey];
		var accruedWithSpecNumValue = jsonAccruedWithSpec[jsonNumKey];
		var allRawValue = jsonAll[jsonPctKey];
		var accruedRawValue = jsonAccrued[jsonPctKey];
		var accruedWithSpecRawValue = jsonAccruedWithSpec[jsonPctKey];
		var allPctValue = parseFloat(allRawValue).toFixed(2);
		var accruedPctValue = parseFloat(accruedRawValue).toFixed(2);
		var accruedWithSpecPctValue = parseFloat(accruedWithSpecRawValue).toFixed(2);
		text += '<td class="valuecolumn">' + allPctValue + '% (' + allNumValue + ')' + '</td>';
		text += '<td class="valuecolumn">' + accruedPctValue + '% (' + accruedNumValue + ')' + '</td>';
		text += '<td class="valuecolumn">' + accruedWithSpecPctValue + '% (' + accruedWithSpecNumValue + ')' + '</td>';
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.createVarStatTableRow3 = function(title, jsonAll, jsonAccrued, jsonAccruedWithSpec, jsonExclPctKey, jsonPctKey, jsonNumKey, status, alertType, cmt)
	{
		// Table row
		var text = '';
		text += '<tr>';
		text += '<td class="propertycolumn">' + title + '</td>';
		var allNumValue = jsonAll[jsonNumKey];
		var accruedNumValue = jsonAccrued[jsonNumKey];
		var accruedWithSpecNumValue = jsonAccruedWithSpec[jsonNumKey];
		var allRawValue = jsonAll[jsonPctKey];
		var accruedRawValue = jsonAccrued[jsonPctKey];
		var accruedWithSpecRawValue = jsonAccruedWithSpec[jsonPctKey];
		var allExclRawValue = jsonAll[jsonExclPctKey];
		var accruedExclRawValue = jsonAccrued[jsonExclPctKey];
		var accruedWithSpecExclRawValue = jsonAccruedWithSpec[jsonExclPctKey];
		var allPctValue = parseFloat(allRawValue).toFixed(2);
		var accruedPctValue = parseFloat(accruedRawValue).toFixed(2);
		var accruedWithSpecPctValue = parseFloat(accruedWithSpecRawValue).toFixed(2);
		var allExclPctValue = parseFloat(allExclRawValue).toFixed(2);
		var accruedExclPctValue = parseFloat(accruedExclRawValue).toFixed(2);
		var accruedWithSpecExclPctValue = parseFloat(accruedWithSpecExclRawValue).toFixed(2);
		text += '<td class="valuecolumn">' + '<b>' + allExclPctValue + '%</b> ' + allPctValue + '% (' + allNumValue + ')' + '</td>';
		text += '<td class="valuecolumn">' + '<b>' + accruedExclPctValue + '%</b> ' + accruedPctValue + '% (' + accruedNumValue + ')' + '</td>';
		text += '<td class="valuecolumn">' + '<b>' + accruedWithSpecExclPctValue + '%</b> ' + accruedWithSpecPctValue + '% (' + accruedWithSpecNumValue + ')' + '</td>';
		if (status != null && alertType != null)
		{
			var statusTag = istat.fetchValidityIcon(status, alertType);
			text += '<td class="statuscolumn">' + statusTag + '</td>';
		}
		text += '<td>' + cmt + '</td>';
		text += '</tr>';
		return text;
	}

	istat.checkForReportFile = function(reportType)
	{
		var frm = document.forms['reggie'];

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=CheckForIncaReportFile';
		url += '&reporttype='+reportType;
		Wizard.showLoadingAnimation('Checking for report file...', 'inca-stat-progress');
		Wizard.asyncJsonRequest(url, istat.reportFileDownloadButtonDisplay);
	}

	istat.reportFileDownloadButtonDisplay = function(response)
	{
		var frm = document.forms['reggie'];

		var reportType = response.reporttype;
		var reportFileExists = response.incaReportFileExists;

		if (reportType == REPORT_TYPE_STATISTICS)
		{
			if (reportFileExists != null && reportFileExists)
			{
				Doc.show('downloadreportfile');
				Doc.removeClass('downloadreportfile', 'disabled');
			}
			else
			{
				Doc.hide('downloadreportfile');
				Doc.addClass('downloadreportfile', 'disabled');
			}
		}
		else
		{
			if (reportFileExists != null && reportFileExists)
			{
				Doc.show('downloadstatcsvfile');
				Doc.removeClass('downloadstatcsvfile', 'disabled');
			}
			else
			{
				Doc.hide('downloadstatcsvfile');
				Doc.addClass('downloadstatcsvfile', 'disabled');
			}
		}
	}

	istat.downloadReportFile = function()
	{
		var frm = document.forms['reggie'];

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadIncaReportFile';
		url += '&reporttype='+REPORT_TYPE_STATISTICS;
		window.open(url);
	}

	istat.downloadStatCsvFile = function()
	{
		var frm = document.forms['reggie'];

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadIncaReportFile';
		url += '&reporttype='+REPORT_TYPE_STATISTICS_CSV;
		window.open(url);
	}

	return istat;
}();

Doc.onLoad(Istat.initPage);

